import { ComponentType, ReactNode } from "react";
import { Context as ReactContext } from "react";
/**
 * Provider id type.
 * Using the id `null` specifies the closest Provider.
 */
type Id = string | number | symbol | null;
/** @internal */
interface Environment<T> {
    has(key: Id): boolean;
    get(key: Id): T | undefined;
    [Symbol.iterator](): IterableIterator<[
        Id,
        T
    ]>;
}
interface ContextConsumerProps<T> {
    /**
     * Which ContextProvider to use
     * @default null
     */
    id?: Id;
    children: (value: T) => ReactNode;
}
type Props = ContextConsumerProps;
type ContextConsumer<T> = ComponentType<Props<T>>;
interface ContextProviderProps<T> {
    /**
     * A value to provide
     */
    value: T;
    /**
     * A key that allows nested Providers to be used
     * @default null
     */
    id?: Id;
    children?: ReactNode;
}
type Props = ContextProviderProps;
type ContextProvider<T> = ComponentType<Props<T>>;
/**
 * A privately scoped unique symbol for accessing Context internal Environment
 * @internal
 */
declare const kEnvironment: unique symbol;
/**
 * A Context with support for multiple keyed values
 */
interface Context$0<T> {
    Consumer: ContextConsumer<T>;
    Provider: ContextProvider<T>;
    /** @internal */
    [kEnvironment]: ReactContext<Environment<T>>;
}
/**
 * Creates a keyed Context allowing multiple nested Providers to be accessible in the same scope
 * @param defaultValue the value consumed if no Provider is in scope and the consumer `id` is
 * `null`
 */
declare function createContext<T>(defaultValue: T): Context$0<T>;
/**
 * Consumes a value from a ContextProvider
 * @param context which Context to use
 * @param id which ContextProvider to use
 */
declare function useContext<T>(context: Context$0<T>, id?: Id): T;
interface Handler<TRequest, TResponse> {
    (request: TRequest, id: Id): PromiseLike<TResponse>;
}
/** @internal */
interface Resource<TResponse> {
    (): TResponse;
}
interface ServiceConsumerProps<TResponse> {
    /**
     * Which ServiceProvider to use
     * @default null
     */
    id?: Id;
    children: (value: TResponse) => ReactNode;
}
type Props = ServiceConsumerProps;
type ServiceConsumer<TResponse> = ComponentType<Props<TResponse>>;
interface ServiceProviderProps<TRequest> {
    /**
     * A request passed to `useHandler()` for fetching an asynchronous resource.
     */
    value: TRequest;
    /**
     * A key that allows nested Providers to be consumed
     * @default null
     */
    id?: Id;
    children?: ReactNode;
    /**
     * A fallback to render if any children are suspended.
     * If the fallback is `null`, `undefined`, or omitted, then a Suspense
     * component must be inserted elsewhere between the Provider and Consumer.
     * @default null
     */
    fallback?: NonNullable<ReactNode> | null;
}
type Props = ServiceProviderProps;
type ServiceProvider<TRequest> = ComponentType<Props<TRequest>>;
/**
 * A privately scoped unique symbol for accessing Service internal Resource
 * @internal
 */
declare const kResource: unique symbol;
/**
 * A Suspense integration for providing asynchronous data through a Context API
 */
interface Service<TRequest, TResponse> {
    Consumer: ServiceConsumer<TResponse>;
    Provider: ServiceProvider<TRequest>;
    /** @internal */
    [kResource]: Context$0<Resource<TResponse>>;
}
/**
 * Creates a Service Context for providing asynchronous data
 * @param handler an asynchronous function for fetching data
 */
declare function createService<TRequest, TResponse>(handler: Handler<TRequest, TResponse>): Service<TRequest, TResponse>;
/**
 * Synchronously consumes a response from a ServiceProvider
 * @param service which Service to use
 * @param id which ServiceProvider to use
 */
declare function useService<TResponse>(service: Service<any, TResponse>, id?: Id): TResponse;
export { Context$0 as Context, Service, createContext, createService, useContext, useService };
//# sourceMappingURL=suspense-service.d.ts.map